import numpy as np
from scipy.interpolate import griddata
import matplotlib.pyplot as plt
import numpy.ma as ma
from numpy.random import uniform, seed
import os
from matplotlib.colors import hsv_to_rgb
import colorsys
from skimage import measure
import matplotlib.ticker as ticker
import matplotlib.font_manager as fm
from colour import Color
import matplotlib.gridspec as gridspec
import math
from matplotlib.pyplot import figure, show, grid, tight_layout
"""
======================
unicode symbols:
======================
"""
alpha_uni = '$\u03B1$'

"""
======================
Definitions
======================
"""
def cm2inch(value):
    return value/2.54

Aalto_yellow = '#FECB00'
Aalto_orange = '#FF7900'
Aalto_red = '#ED2939'
Aalto_violet = '#B1059D'
Aalto_purple = '#6639B7'
Aalto_blue = '#0065BD'
Aalto_turqoise = '#00A8B4'
Aalto_green = '#009B3A'
Aalto_light_green = '#69BE28'
Aalto_grey = '#928B81'





"""
======================
1. input file
======================
"""
file1 = 'A_10ML_Ni.txt'
file2 = 'B_10ML_Ni_annealed.txt'
file3 = 'C_10ML_Ni_field1.txt'
file4 = 'D_10ML_Ni_field2.txt'
file5 = 'D_10ML_Ni_field2_c.txt'

headerrows = 2



"""
2. define plotting parameters:
"""


'''
#size of square plot:
x_pixels = 1000
y_pixels = 900
#set a phase to rotate the colourwheel in degrees
phase = 180
#set saturation and value of the colours
saturation = 0.6
value = 1
#choose font
# parameters related to the data
para1_min = 0.0
para1_max = 5e6
para1_tick_number = 5
para1_exponent = 6
para1_label = '$\mathit{K_{pp}}$ [x 10$^'+ str(para1_exponent) +'$ J/m$^{3}$]'
para1_altlabel = '$\mathit{K_{pp,eff}}$ [x 10$^'+ str(para1_exponent) +'$ J/m$^{3}$]'
para1_altshift = 628318.53
para2_min = 0.0
para2_max = 6e4
para2_tick_number = 6
para2_exponent = 4
para2_label = '$\mathit{K_{ip}}$ [x 10$^'+ str(para2_exponent) +'$ J/m$^{3}$]'
para2_altlabel = '$\mathit{K_{ip}^{2}}$ [x 10$^'+ str(para2_exponent*2) +'$ J$^{2}$/m$^{6}$]'
'''

label_fontsize = 8
tick_fontsize = 6

contourlinewidth = 0.5
plotlinewidth = 1
symbolsize = 3

x1 = 0.17
x2 = 0.61
y1 = 0.1
y2 = 0.6
panelheight = 0.31
panelwidth = 0.4

# define panel size and position:
width = 8.6
heightratio = 1
figuresize=(cm2inch(width), cm2inch(heightratio*width))
panel_1 = [x1, y2, panelheight, panelwidth]
panel_2 = [x2, y2, panelheight, panelwidth]
#panel_3 = [0.61, 0.5, 0.31, 0.35]
panel_4 = [x1, y1, panelheight, panelwidth]
panel_5 = [x2, y1, panelheight, panelwidth]

rlabelpad = -1
thetalabelpad = -3
xlabelpad = 1
ylabelpad = -1

rlabelpos = (-0.5,0.5)
Deltalabelpos =(0.85,-0.15)


"""
create image:
"""

fig = plt.figure(figsize=figuresize, dpi=80, facecolor='w', edgecolor='k')

"""
add panel labels
"""
axlabels = fig.add_axes([0,0,1,1],label ='labels')
axlabels.axis('off')
axlabels.patch.set_alpha(0.0)
axlabels.text(0.01,0.98,'(a)',fontsize=label_fontsize)
axlabels.text(0.54,0.98,'(b)',fontsize=label_fontsize)
axlabels.text(0.01,0.48,'(c)',fontsize=label_fontsize)
axlabels.text(0.54,0.48,'(d)',fontsize=label_fontsize)
axlabels.tick_params(bottom=False, top=False, left=False, right=False, labelbottom=False, labeltop=False, labelleft=False, labelright=False)

"""
polar plot
"""
m1angle = np.loadtxt(file1,skiprows=headerrows)
m2angle = np.loadtxt(file2,skiprows=headerrows)
m3angle = np.loadtxt(file3,skiprows=headerrows)
m4angle = np.loadtxt(file4,skiprows=headerrows)
m5angle = np.loadtxt(file5,skiprows=headerrows)

theta1 = m1angle[:,0]
theta2 = m2angle[:,0]
theta3 = m3angle[:,0]
theta4 = m4angle[:,0]
theta5 = m5angle[:,0]

A_a = m1angle[:,1] 
A_b = m1angle[:,2]
B_a = m2angle[:,5]
B_b = m2angle[:,2]
B_c = m2angle[:,6]
B_d = m2angle[:,4]
C_a = m3angle[:,1]
C_b = m3angle[:,2]
D_a = m4angle[:,1]
D_b = m4angle[:,2]
E_c = m5angle[:,1]

theory_angles = np.arange(0,361)
theory_A_a = np.zeros(theory_angles.size)
theory_A_b = np.zeros(theory_angles.size)
theory_B_a = np.zeros(theory_angles.size)
theory_B_b = np.zeros(theory_angles.size)
theory_B_c = np.zeros(theory_angles.size)
theory_B_d = np.zeros(theory_angles.size)
theory_C_a = np.zeros(theory_angles.size)
theory_C_b = np.zeros(theory_angles.size)
theory_D_a = np.zeros(theory_angles.size)
theory_D_b = np.zeros(theory_angles.size)
theory_E_c = np.zeros(theory_angles.size)
zero_line = np.zeros(theory_angles.size)
 
A_a_direction = 248.96
A_a_error = 2.15
A_b_direction = 69.33
A_b_error = 3.62
B_a_direction = 138.24
B_a_error = 2.77
B_b_direction = 306.94
B_b_error = 3.67
B_c_direction = 80.3
B_c_error = 4.61
B_d_direction = 248.27
B_d_error = 3.04
C_a_direction = 305.71
C_a_error = 1.71
C_b_direction = 251.46
C_b_error = 2.1
D_a_direction = 303.52
D_a_error = 2.59
D_b_direction = 249.12
D_b_error = 4.75
E_c_direction = 96.86
E_c_error = 13.72

stripe_direction = 281.20
#stripe_direction = 0


"""
calculate deltas:
"""

delta_A_ab = int(round(A_a_direction - A_b_direction,0))
delta_B_ab = int(round(B_b_direction - B_a_direction,0))
delta_B_cd = int(round(B_d_direction - B_c_direction,0))
delta_C_ab = int(round(C_a_direction - C_b_direction,0))
delta_D_ab = int(round(D_a_direction - D_b_direction,0))
delta_Da_Ec = int(round(360-(D_a_direction - E_c_direction),0))


"""
adjust and calculate errors:
"""

errorfactor = 2

A_a_error = A_a_error*errorfactor 
A_b_error = A_b_error*errorfactor
B_a_error = B_a_error*errorfactor
B_b_error = B_b_error*errorfactor
B_c_error = B_c_error*errorfactor
B_d_error = B_d_error*errorfactor
C_a_error = C_a_error*errorfactor
C_b_error = C_b_error*errorfactor
D_a_error = D_a_error*errorfactor
D_b_error = D_b_error*errorfactor
E_c_error = E_c_error

delta_A_ab_error = int(round(math.sqrt(A_a_error**2+A_b_error**2),0))
delta_B_ab_error = int(round(math.sqrt(B_a_error**2+B_b_error**2),0))
delta_B_cd_error = int(round(math.sqrt(B_c_error**2+B_d_error**2),0))
delta_C_ab_error = int(round(math.sqrt(C_a_error**2+C_b_error**2),0))
delta_D_ab_error = int(round(math.sqrt(D_a_error**2+D_b_error**2),0))
delta_Da_Ec_error = int(round(math.sqrt(D_a_error**2+E_c_error**2),0))



"""
rescale data for correct zero:
"""
theta1_renorm = theta1 - stripe_direction
theta2_renorm = theta2 - stripe_direction
theta3_renorm = theta3 - stripe_direction
theta4_renorm = theta4 - stripe_direction
theta5_renorm = theta5 - stripe_direction

A_a_direction_renorm = A_a_direction - stripe_direction
A_b_direction_renorm = A_b_direction - stripe_direction
B_a_direction_renorm = B_a_direction - stripe_direction
B_b_direction_renorm = B_b_direction - stripe_direction
B_c_direction_renorm = B_c_direction - stripe_direction
B_d_direction_renorm = B_d_direction - stripe_direction
C_a_direction_renorm = C_a_direction - stripe_direction
C_b_direction_renorm = C_b_direction - stripe_direction
D_a_direction_renorm = D_a_direction - stripe_direction
D_b_direction_renorm = D_b_direction - stripe_direction
E_c_direction_renorm = E_c_direction - stripe_direction

"""
create theory curves:
"""

for n in range(0,theory_angles.size):
    theory_A_a[n] = math.cos((theory_angles[n]-A_a_direction_renorm)/180*math.pi)
    theory_A_b[n] = math.cos((theory_angles[n]-A_b_direction_renorm)/180*math.pi)
    theory_B_a[n] = math.cos((theory_angles[n]-B_a_direction_renorm)/180*math.pi)
    theory_B_b[n] = math.cos((theory_angles[n]-B_b_direction_renorm)/180*math.pi)
    theory_B_c[n] = math.cos((theory_angles[n]-B_c_direction_renorm)/180*math.pi)
    theory_B_d[n] = math.cos((theory_angles[n]-B_d_direction_renorm)/180*math.pi)
    theory_C_a[n] = math.cos((theory_angles[n]-C_a_direction_renorm)/180*math.pi)
    theory_C_b[n] = math.cos((theory_angles[n]-C_b_direction_renorm)/180*math.pi)
    theory_D_a[n] = math.cos((theory_angles[n]-D_a_direction_renorm)/180*math.pi)
    theory_D_b[n] = math.cos((theory_angles[n]-D_b_direction_renorm)/180*math.pi)
    theory_E_c[n] = math.cos((theory_angles[n]-E_c_direction_renorm)/180*math.pi)



"""
A_10ML_Ni
"""
ax1 = fig.add_axes(panel_1, label ='1', projection='polar')
ax1.plot(theory_angles/180*math.pi, theory_A_a, color =  Aalto_blue, linewidth=plotlinewidth)
ax1.plot((theta1_renorm)/180*math.pi, A_a,'o', markersize= symbolsize, color =  Aalto_blue)
ax1.plot((-1.35, A_a_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_blue, linewidth=2*plotlinewidth)
ax1.plot(theory_angles/180*math.pi, theory_A_b, color =  Aalto_orange, linewidth=plotlinewidth)
ax1.plot(theta1_renorm/180*math.pi, A_b,'o', markersize= symbolsize, color =  Aalto_orange)
ax1.plot((-1.35, A_b_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_orange, linewidth=2*plotlinewidth)
ax1.plot(theory_angles/180*math.pi, zero_line, color =  'black', linewidth=plotlinewidth)

ax1.bar(A_a_direction_renorm*np.pi/180,1.35, width = A_a_error*np.pi/90, bottom=-1.35, color = Aalto_blue, alpha= 0.25)
ax1.bar(A_a_direction_renorm*np.pi/180,1.35, width = A_a_error*np.pi/90, bottom=0, color = Aalto_blue, alpha= 0.25)
ax1.bar(A_b_direction_renorm*np.pi/180,1.35, width = A_b_error*np.pi/90, bottom=-1.35, color = Aalto_orange, alpha= 0.25)
ax1.bar(A_b_direction_renorm*np.pi/180,1.35, width = A_b_error*np.pi/90, bottom=0, color = Aalto_orange, alpha= 0.25)

ax1.set_theta_zero_location("S",offset=stripe_direction)
ax1.set_xticks([0,30*np.pi/180.,60*np.pi/180.,90*np.pi/180.,120*np.pi/180.,150*np.pi/180.,180*np.pi/180.,210*np.pi/180.,240*np.pi/180.,270*np.pi/180.,300*np.pi/180.,330*np.pi/180.])
#ax1.set_xticklabels(['0','30','60','90','120','150','180','210','240','270','300','330'])
ax1.set_ylim(-1.3500000000000001, 1.35)
ax1.set_yticklabels([])
ax1.set_yticks([-1,0,1])


###
# move r-axis out of the plot
###
X_OFFSET = 0.08 # to control how far the scale is from the plot (axes coordinates)
rect = ax1.get_position()
rect = (rect.xmin-X_OFFSET, rect.ymin+rect.height/2, # x, y
        rect.width, rect.height/2) # width, height
scale_ax1 = ax1.figure.add_axes(rect)
# hide most elements of the new axes
for loc in ['right', 'top', 'bottom']:
    scale_ax1.spines[loc].set_visible(False)
scale_ax1.tick_params(bottom=False, labelbottom=False)
scale_ax1.patch.set_visible(False) # hide white background
# adjust the scale
scale_ax1.spines['left'].set_bounds(*ax1.get_ylim())
# scale_ax1.spines['left'].set_bounds(0, ax.get_rmax()) # mpl < 2.2.3
scale_ax1.set_yticks(ax1.get_yticks())
scale_ax1.set_ylim(ax1.get_rorigin(), ax1.get_rmax())
# scale_ax1.set_ylim(ax.get_ylim()) # Matplotlib < 2.2.3
rect2 = ax1.get_position()
rect2 = (rect2.xmin-X_OFFSET, rect2.ymin, # x, y
        rect2.width, rect2.height/2) # width, height
scale_ax11 = ax1.figure.add_axes(rect2)
# hide most elements of the new axes
for loc in ['right', 'top', 'bottom']:
    scale_ax11.spines[loc].set_visible(False)
scale_ax11.tick_params(bottom=False, labelbottom=False)
scale_ax11.patch.set_visible(False) # hide white background
# adjust the scale
scale_ax11.spines['left'].set_bounds(*ax1.get_ylim())
scale_ax11.set_yticks(ax1.get_yticks())
scale_ax11.set_ylim(ax1.get_rmax(),ax1.get_rorigin())

ax1.tick_params(labelsize=tick_fontsize)
scale_ax1.tick_params(direction='in',labelsize=tick_fontsize)
scale_ax11.tick_params(direction='in',labelsize=tick_fontsize)
ax1.tick_params(axis='x', which='major', pad=thetalabelpad)

ax1.annotate('m',xy = rlabelpos, xycoords = 'axes fraction', ha='center', va='center', rotation=90, fontsize=label_fontsize)

ax1.annotate('$\Delta=$'+str(delta_A_ab) +'$\pm$' + str(delta_A_ab_error),xy =Deltalabelpos,xycoords = 'axes fraction', ha='center', va='top', rotation=0, fontsize=label_fontsize)



"""
B_10ML_Ni_annealed, a & b domains
"""
ax2 = fig.add_axes(panel_2, label ='2', projection='polar')
ax2.plot(theory_angles/180*math.pi, theory_B_a, color =  Aalto_orange, linewidth=plotlinewidth)
ax2.plot(theta2_renorm/180*math.pi, B_a,'o', markersize= symbolsize, color =  Aalto_orange)
ax2.plot((-1.35, B_a_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_orange, linewidth=2*plotlinewidth)
ax2.plot(theory_angles/180*math.pi, theory_B_b, color =  Aalto_violet, linewidth=plotlinewidth)
ax2.plot(theta2_renorm/180*math.pi, B_b,'o', markersize= symbolsize, color =  Aalto_violet)
ax2.plot((-1.35, B_b_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_violet, linewidth=2*plotlinewidth)
ax2.plot(theory_angles/180*math.pi, zero_line, color =  'black', linewidth=plotlinewidth)

ax2.bar(B_a_direction_renorm*np.pi/180,1.35, width = B_a_error*np.pi/90, bottom=-1.35, color = Aalto_orange, alpha= 0.25)
ax2.bar(B_a_direction_renorm*np.pi/180,1.35, width = B_a_error*np.pi/90, bottom=0, color = Aalto_orange, alpha= 0.25)
ax2.bar(B_b_direction_renorm*np.pi/180,1.35, width = B_b_error*np.pi/90, bottom=-1.35, color = Aalto_violet, alpha= 0.25)
ax2.bar(B_b_direction_renorm*np.pi/180,1.35, width = B_b_error*np.pi/90, bottom=0, color = Aalto_violet, alpha= 0.25)

ax2.set_theta_zero_location("S",offset=stripe_direction)
ax2.set_xticks([0,30*np.pi/180.,60*np.pi/180.,90*np.pi/180.,120*np.pi/180.,150*np.pi/180.,180*np.pi/180.,210*np.pi/180.,240*np.pi/180.,270*np.pi/180.,300*np.pi/180.,330*np.pi/180.])
#ax2.set_xticklabels(['0','30','60','90','120','150','180','210','240','270','300','330'])
ax2.set_ylim(-1.3500000000000001, 1.35)
ax2.set_yticklabels([])
ax2.set_yticks([-1,0,1])

ax2.tick_params(labelsize=tick_fontsize)
ax2.tick_params(axis='x', which='major', pad=thetalabelpad)
#ax2.annotate('$\Delta=$'+str(delta_B_ab)+'$\pm$' + str(delta_B_ab_error),xy =Deltalabelpos,xycoords = 'axes fraction', ha='center', va='top', rotation=0, fontsize=label_fontsize)
ax2.annotate('$\Delta_{1}=$'+str(delta_B_ab)+'$\pm$' + str(delta_B_ab_error)+'\n $\Delta_{2}=$'+str(delta_B_cd)+'$\pm$' + str(delta_B_cd_error) ,xy =Deltalabelpos,xycoords = 'axes fraction', ha='center', va='top', rotation=0, fontsize=label_fontsize)


"""
###
# move r-axis out of the plot
###
rect = ax2.get_position()
rect = (rect.xmin-X_OFFSET, rect.ymin+rect.height/2, # x, y
        rect.width, rect.height/2) # width, height
scale_ax2 = ax2.figure.add_axes(rect)
# hide most elements of the new axes
for loc in ['right', 'top', 'bottom']:
    scale_ax2.spines[loc].set_visible(False)
scale_ax2.tick_params(bottom=False, labelbottom=False)
scale_ax2.patch.set_visible(False) # hide white background
# adjust the scale
scale_ax2.spines['left'].set_bounds(*ax2.get_ylim())
# scale_ax1.spines['left'].set_bounds(0, ax.get_rmax()) # mpl < 2.2.3
scale_ax2.set_yticks(ax2.get_yticks())
scale_ax2.set_ylim(ax2.get_rorigin(), ax2.get_rmax())
# scale_ax1.set_ylim(ax.get_ylim()) # Matplotlib < 2.2.3
rect2 = ax2.get_position()
rect2 = (rect2.xmin-X_OFFSET, rect2.ymin, # x, y
        rect2.width, rect2.height/2) # width, height
scale_ax21 = ax2.figure.add_axes(rect2)
# hide most elements of the new axes
for loc in ['right', 'top', 'bottom']:
    scale_ax21.spines[loc].set_visible(False)
scale_ax21.tick_params(bottom=False, labelbottom=False)
scale_ax21.patch.set_visible(False) # hide white background
# adjust the scale
scale_ax21.spines['left'].set_bounds(*ax2.get_ylim())
scale_ax21.set_yticks(ax2.get_yticks())
scale_ax21.set_ylim(ax2.get_rmax(),ax2.get_rorigin())

ax2.tick_params(labelsize=tick_fontsize)
scale_ax2.tick_params(direction='in',labelsize=tick_fontsize)
scale_ax21.tick_params(direction='in',labelsize=tick_fontsize)
ax2.tick_params(axis='x', which='major', pad=thetalabelpad)
ax2.annotate('m',xy = rlabelpos, xycoords = 'axes fraction', ha='center', va='center', rotation=90, fontsize=label_fontsize)
"""


"""
B_10ML_Ni_annealed, c & d domains
"""
#ax3 = fig.add_axes(panel_3, label ='3', projection='polar')
ax2.plot(theory_angles/180*math.pi, theory_B_c, color =  Aalto_blue, linewidth=plotlinewidth)
ax2.plot(theta2_renorm/180*math.pi, B_c,'o', markersize= symbolsize, color =  Aalto_blue)
ax2.plot((-1.35, B_c_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_blue, linewidth=2*plotlinewidth)
ax2.plot(theory_angles/180*math.pi, theory_B_d, color =  Aalto_green, linewidth=plotlinewidth)
ax2.plot(theta2_renorm/180*math.pi, B_d,'o', markersize= symbolsize, color =  Aalto_green)
ax2.plot((-1.35, B_d_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_green, linewidth=2*plotlinewidth)
ax2.plot(theory_angles/180*math.pi, zero_line, color =  'black', linewidth=plotlinewidth)

ax2.bar(B_c_direction_renorm*np.pi/180,1.35, width = B_c_error*np.pi/90, bottom=-1.35, color = Aalto_blue, alpha= 0.25)
ax2.bar(B_c_direction_renorm*np.pi/180,1.35, width = B_c_error*np.pi/90, bottom=0, color = Aalto_blue, alpha= 0.25)
ax2.bar(B_d_direction_renorm*np.pi/180,1.35, width = B_d_error*np.pi/90, bottom=-1.35, color = Aalto_green, alpha= 0.25)
ax2.bar(B_d_direction_renorm*np.pi/180,1.35, width = B_d_error*np.pi/90, bottom=0, color = Aalto_green, alpha= 0.25)

#ax3.set_theta_zero_location("S",offset=stripe_direction)
#ax3.set_xticks([0,30*np.pi/180.,60*np.pi/180.,90*np.pi/180.,120*np.pi/180.,150*np.pi/180.,180*np.pi/180.,210*np.pi/180.,240*np.pi/180.,270*np.pi/180.,300*np.pi/180.,330*np.pi/180.])
#ax3.set_xticklabels(['0','30','60','90','120','150','180','210','240','270','300','330'])
#ax3.set_ylim(-1.3500000000000001, 1.35)
#ax3.set_yticklabels([])
#ax3.set_yticks([-1,0,1])

#ax3.tick_params(labelsize=tick_fontsize)
#ax3.tick_params(axis='x', which='major', pad=thetalabelpad)
#ax3.annotate('$\Delta=$'+str(delta_B_cd)+'$\pm$' + str(delta_B_cd_error),xy =Deltalabelpos,xycoords = 'axes fraction', ha='center', va='top', rotation=0, fontsize=label_fontsize)



"""
C_field1
"""
ax4 = fig.add_axes(panel_4, label ='4', projection='polar')
ax4.plot(theory_angles/180*math.pi, theory_C_a, color =  Aalto_violet, linewidth=plotlinewidth)
ax4.plot(theta3_renorm/180*math.pi, C_a,'o', markersize= symbolsize, color =  Aalto_violet)
ax4.plot((-1.35, C_a_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_violet, linewidth=2*plotlinewidth)
ax4.plot(theory_angles/180*math.pi, theory_C_b, color =  Aalto_green, linewidth=plotlinewidth)
ax4.plot(theta3_renorm/180*math.pi, C_b,'o', markersize= symbolsize, color =  Aalto_green)
ax4.plot((-1.35, C_b_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_green, linewidth=2*plotlinewidth)
ax4.plot(theory_angles/180*math.pi, zero_line, color =  'black', linewidth=plotlinewidth)

ax4.bar(C_a_direction_renorm*np.pi/180,1.35, width = C_a_error*np.pi/90, bottom=-1.35, color = Aalto_violet, alpha= 0.25)
ax4.bar(C_a_direction_renorm*np.pi/180,1.35, width = C_a_error*np.pi/90, bottom=0, color = Aalto_violet, alpha= 0.25)
ax4.bar(C_b_direction_renorm*np.pi/180,1.35, width = C_b_error*np.pi/90, bottom=-1.35, color = Aalto_green, alpha= 0.25)
ax4.bar(C_b_direction_renorm*np.pi/180,1.35, width = C_b_error*np.pi/90, bottom=0, color = Aalto_green, alpha= 0.25)

ax4.set_theta_zero_location("S",offset=stripe_direction)
ax4.set_xticks([0,30*np.pi/180.,60*np.pi/180.,90*np.pi/180.,120*np.pi/180.,150*np.pi/180.,180*np.pi/180.,210*np.pi/180.,240*np.pi/180.,270*np.pi/180.,300*np.pi/180.,330*np.pi/180.])
#ax4.set_xticklabels(['0','30','60','90','120','150','180','210','240','270','300','330'])
ax4.set_ylim(-1.3500000000000001, 1.35)
ax4.set_yticklabels([])
ax4.set_yticks([-1,0,1])

ax4.tick_params(labelsize=tick_fontsize)
ax4.tick_params(axis='x', which='major', pad=thetalabelpad)
ax4.annotate('$\Delta=$'+str(delta_C_ab)+'$\pm$' + str(delta_C_ab_error),xy =Deltalabelpos,xycoords = 'axes fraction', ha='center', va='top', rotation=0, fontsize=label_fontsize)

###
# move r-axis out of the plot
###
rect = ax4.get_position()
rect = (rect.xmin-X_OFFSET, rect.ymin+rect.height/2, # x, y
        rect.width, rect.height/2) # width, height
scale_ax4 = ax4.figure.add_axes(rect)
# hide most elements of the new axes
for loc in ['right', 'top', 'bottom']:
    scale_ax4.spines[loc].set_visible(False)
scale_ax4.tick_params(bottom=False, labelbottom=False)
scale_ax4.patch.set_visible(False) # hide white background
# adjust the scale
scale_ax4.spines['left'].set_bounds(*ax4.get_ylim())
# scale_ax1.spines['left'].set_bounds(0, ax.get_rmax()) # mpl < 2.2.3
scale_ax4.set_yticks(ax4.get_yticks())
scale_ax4.set_ylim(ax4.get_rorigin(), ax4.get_rmax())
# scale_ax1.set_ylim(ax.get_ylim()) # Matplotlib < 2.2.3
rect2 = ax4.get_position()
rect2 = (rect2.xmin-X_OFFSET, rect2.ymin, # x, y
        rect2.width, rect2.height/2) # width, height
scale_ax41 = ax4.figure.add_axes(rect2)
# hide most elements of the new axes
for loc in ['right', 'top', 'bottom']:
    scale_ax41.spines[loc].set_visible(False)
scale_ax41.tick_params(bottom=False, labelbottom=False)
scale_ax41.patch.set_visible(False) # hide white background
# adjust the scale
scale_ax41.spines['left'].set_bounds(*ax4.get_ylim())
scale_ax41.set_yticks(ax4.get_yticks())
scale_ax41.set_ylim(ax4.get_rmax(),ax4.get_rorigin())

ax4.tick_params(labelsize=tick_fontsize)
scale_ax4.tick_params(direction='in',labelsize=tick_fontsize)
scale_ax41.tick_params(direction='in',labelsize=tick_fontsize)
ax4.tick_params(axis='x', which='major', pad=thetalabelpad)
ax4.annotate('m',xy = rlabelpos, xycoords = 'axes fraction', ha='center', va='center', rotation=90, fontsize=label_fontsize)


"""
D_E_field2
"""
ax5 = fig.add_axes(panel_5, label ='5', projection='polar')
ax5.plot(theory_angles/180*math.pi, theory_D_a, color =  Aalto_violet, linewidth=plotlinewidth)
ax5.plot(theta4_renorm/180*math.pi, D_a,'o', markersize= symbolsize, color =  Aalto_violet)
ax5.plot((-1.35, D_a_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_violet, linewidth=2*plotlinewidth)
ax5.plot(theory_angles/180*math.pi, theory_D_b, color =  Aalto_green, linewidth=plotlinewidth)
ax5.plot(theta4_renorm/180*math.pi, D_b,'o', markersize= symbolsize, color =  Aalto_green)
ax5.plot((-1.35, D_b_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_green, linewidth=2*plotlinewidth)
ax5.plot(theory_angles/180*math.pi, theory_E_c, color =  Aalto_blue, linewidth=plotlinewidth)
ax5.plot(theta5_renorm/180*math.pi, E_c,'o', markersize= symbolsize, color =  Aalto_blue)
ax5.plot((-1.35, E_c_direction_renorm/180*math.pi), ( -1.35, 1.35), color =  Aalto_blue, linewidth=2*plotlinewidth)
ax5.plot(theory_angles/180*math.pi, zero_line, color =  'black', linewidth=plotlinewidth)

ax5.bar(D_a_direction_renorm*np.pi/180,1.35, width = D_a_error*np.pi/90, bottom=-1.35, color = Aalto_violet, alpha= 0.25)
ax5.bar(D_a_direction_renorm*np.pi/180,1.35, width = D_a_error*np.pi/90, bottom=0, color = Aalto_violet, alpha= 0.25)
ax5.bar(D_b_direction_renorm*np.pi/180,1.35, width = D_b_error*np.pi/90, bottom=-1.35, color = Aalto_green, alpha= 0.25)
ax5.bar(D_b_direction_renorm*np.pi/180,1.35, width = D_b_error*np.pi/90, bottom=0, color = Aalto_green, alpha= 0.25)
ax5.bar(E_c_direction_renorm*np.pi/180,1.35, width = E_c_error*np.pi/90, bottom=-1.35, color = Aalto_blue, alpha= 0.25)
ax5.bar(E_c_direction_renorm*np.pi/180,1.35, width = E_c_error*np.pi/90, bottom=0, color = Aalto_blue, alpha= 0.25)

ax5.set_theta_zero_location("S",offset=stripe_direction)
ax5.set_xticks([0,30*np.pi/180.,60*np.pi/180.,90*np.pi/180.,120*np.pi/180.,150*np.pi/180.,180*np.pi/180.,210*np.pi/180.,240*np.pi/180.,270*np.pi/180.,300*np.pi/180.,330*np.pi/180.])
#ax5.set_xticklabels(['0','30','60','90','120','150','180','210','240','270','300','330'])
ax5.set_ylim(-1.3500000000000001, 1.35)
ax5.set_yticklabels([])
ax5.set_yticks([-1,0,1])

ax5.tick_params(labelsize=tick_fontsize)
ax5.tick_params(axis='x', which='major', pad=thetalabelpad)
ax5.annotate('$\Delta_{1}=$'+str(delta_D_ab)+'$\pm$' + str(delta_D_ab_error)+'\n $\Delta_{2}=$'+str(delta_Da_Ec)+'$\pm$' + str(delta_Da_Ec_error) ,xy =Deltalabelpos,xycoords = 'axes fraction', ha='center', va='top', rotation=0, fontsize=label_fontsize)

"""
show or save image
"""  
#plt.show()
plt.savefig('Angular_dependences_BTO_4s4.pdf', bbox_inches = 'tight',pad_inches = 0)
plt.clf()



