import numpy as np
from scipy.interpolate import griddata
import matplotlib.pyplot as plt
import numpy.ma as ma
from numpy.random import uniform, seed
import os
from matplotlib.colors import hsv_to_rgb
import colorsys
from skimage import measure
import matplotlib.ticker as ticker
import matplotlib.font_manager as fm
from colour import Color
import matplotlib.gridspec as gridspec
import math
from matplotlib.pyplot import figure, show, grid, tight_layout
from PIL import Image
import scipy.ndimage
import matplotlib.image as mpimg
import matplotlib.patches as patches




"""
======================
Definitions
======================
"""
def cm2inch(value):
    return value/2.54

Aalto_yellow = '#FECB00' 
Aalto_orange = '#FF7900' #
Aalto_red = '#ED2939' 
Aalto_violet = '#B1059D' #
Aalto_purple = '#6639B7'
Aalto_blue = '#0065BD' #
Aalto_turqoise = '#00A8B4'
Aalto_green = '#009B3A' #
Aalto_light_green = '#69BE28'
Aalto_grey = '#928B81'

nmpp = 21.6 #This is nm per image pixel

"""
2. define parameters:
"""

headerrows = 3


label_fontsize = 8
tick_fontsize = 6

contourlinewidth = 0.5
plotlinewidth = 1
symbolsize = 3

# define panel size and position:
width = 8.6
heightratio = 0.97
figuresize=(cm2inch(width), cm2inch(heightratio*width))
panelwidth = 0.38
panelheight = 0.27
imagexfactor = 1.05
imageyshift = -0.365
imagexshift = 0.00

panel_1 = [0.51, 0.72, panelwidth, panelheight]
panel_2 = [0.51, 0.4, panelwidth, panelheight]
panel_3 = [0.51, 0.08, panelwidth, panelheight]
panel_4 = [0.11+imagexshift, 0.72+imageyshift, panelwidth*imagexfactor, 1]
panel_5 = [0.11, 0.4, panelwidth, panelheight]
panel_6 = [0.11, 0.08, panelwidth, panelheight]

panel_test = [0.11, 0.72, panelwidth, panelheight]

rlabelpad = -1
thetalabelpad = -3
xlabelpad = 1
ylabelpad = 1

xScaling = 1e3

"""
create image:
"""
fig = plt.figure(num=None, figsize=figuresize, dpi=80, facecolor='w', edgecolor='k')
ax1 = fig.add_axes(panel_1, label ='1')
ax2 = fig.add_axes(panel_2, label ='2')
ax3 = fig.add_axes(panel_3, label ='3')
ax4 = fig.add_axes(panel_4, label ='4')
ax5 = fig.add_axes(panel_5, label ='5')
ax6 = fig.add_axes(panel_6, label ='6')

'''
axtest = fig.add_axes(panel_test, label ='44')
axtest.patch.set_alpha(0.0)
axtest.tick_params(bottom=False, top=False, left=False, right=False, labelbottom=False, labeltop=False, labelleft=False, labelright=False)
'''
"""
======================
import data
======================
"""
domain_angles = np.loadtxt('domain_angles.txt',skiprows=headerrows)

"""
======================
plot data
======================
"""

ax1.fill_between((domain_angles[:,1])/xScaling, domain_angles[:,3], domain_angles[:,4], color = Aalto_grey, alpha = 0.7)
ax1.plot((domain_angles[:,1])/xScaling, domain_angles[:,2], color = 'black', linewidth=plotlinewidth)
ax1.plot((domain_angles[:,1])/xScaling, domain_angles[:,5], color = Aalto_red, linewidth=plotlinewidth)
ax1.plot((domain_angles[:,1])/xScaling, domain_angles[:,6], color = Aalto_turqoise, linewidth=plotlinewidth)

ax2.fill_between((domain_angles[:,7])/xScaling, domain_angles[:,9], domain_angles[:,10], color = Aalto_grey, alpha = 0.7)
ax2.plot((domain_angles[:,7])/xScaling, domain_angles[:,8], color = 'black', linewidth=plotlinewidth)
ax2.plot((domain_angles[:,7])/xScaling, domain_angles[:,11], color = Aalto_red, linewidth=plotlinewidth)
ax2.plot((domain_angles[:,7])/xScaling, domain_angles[:,12], color = Aalto_turqoise, linewidth=plotlinewidth)

ax3.fill_between((domain_angles[:,13])/xScaling, domain_angles[:,15], domain_angles[:,16], color = Aalto_grey, alpha = 0.7)
ax3.plot((domain_angles[:,13])/xScaling, domain_angles[:,14], color = 'black', linewidth=plotlinewidth)
ax3.plot((domain_angles[:,13])/xScaling, domain_angles[:,17], color = Aalto_red, linewidth=plotlinewidth)
ax3.plot((domain_angles[:,13])/xScaling, domain_angles[:,18], color = Aalto_turqoise, linewidth=plotlinewidth)



"""
======================
plot design
======================
"""

ax1.set_xlim(np.amin(domain_angles[:,1])/xScaling, np.amax(domain_angles[:,1])/xScaling)
ax2.set_xlim(np.amax(domain_angles[:,7])/xScaling, np.amin(domain_angles[:,7])/xScaling)
ax3.set_xlim(np.amin(domain_angles[:,13])/xScaling, np.amax(domain_angles[:,13])/xScaling)

ax1.set_ylim(0, 20)
ax2.set_ylim(-30, -10)
ax3.set_ylim(15, 35)


ax1.xaxis.set_major_locator(plt.MultipleLocator(0.05))
ax2.xaxis.set_major_locator(plt.MultipleLocator(0.1))
ax3.xaxis.set_major_locator(plt.MultipleLocator(0.05))
ax1.yaxis.set_major_locator(plt.MultipleLocator(10))
ax2.yaxis.set_major_locator(plt.MultipleLocator(10))
ax3.set_yticks([15,25,35])

ax3.set_xlabel('$w$ [$\mathrm{\mu}$m] ', fontsize = label_fontsize, labelpad=xlabelpad)
ax1.set_ylabel('$\phi_{\mathrm{1}}$ [deg] ', fontsize = label_fontsize, labelpad=ylabelpad)
ax2.set_ylabel('$\phi_{\mathrm{2}}$ [deg] ', fontsize = label_fontsize, labelpad=ylabelpad)
ax3.set_ylabel('$\phi_{\mathrm{3}}$ [deg] ', fontsize = label_fontsize, labelpad=ylabelpad)
ax1.yaxis.set_label_position("right")
ax2.yaxis.set_label_position("right")
ax3.yaxis.set_label_position("right")

# position ticks
ax1.tick_params(bottom=True, top=False, left=True, right=True, labelbottom=True, labeltop=False, labelleft=False, labelright=True, direction='in', labelsize=tick_fontsize)
ax2.tick_params(bottom=True, top=False, left=True, right=True, labelbottom=True, labeltop=False, labelleft=False, labelright=True, direction='in', labelsize=tick_fontsize)
ax3.tick_params(bottom=True, top=False, left=True, right=True, labelbottom=True, labeltop=False, labelleft=False, labelright=True, direction='in', labelsize=tick_fontsize)



###
#Add fitted images
###

"""
======================
parameters
======================
"""

profileRotation = 11.20 #this is the average of the DW orientations
profile_location1 = 185
profile_location2 = 515

NewWidth = 450
NewHeight = 320
WidthShift = 8
HeightShift = -4

"""
======================
define functions
======================
"""

def rgb2gray(rgb):
    return np.dot(rgb[...,:3], [0.2989, 0.5870, 0.1140])


def rotate(origin, point, angle):
    """
    Rotate a point counterclockwise by a given angle around a given origin.

    The angle should be given in radians.
    """
    ox, oy = origin
    px, py = point

    qx = ox + math.cos(angle) * (px - ox) - math.sin(angle) * (py - oy)
    qy = oy + math.sin(angle) * (px - ox) + math.cos(angle) * (py - oy)
    return qx, qy

"""
======================
import image
======================
"""

img = mpimg.imread('SPLEEM_02_data_fit_absolute_range_-5.6e+00_3.6e+00.png')
height, width, depth= img.shape

#mask image with square
img = img[int((height-NewHeight)/2+HeightShift):int((height+NewHeight)/2+HeightShift),int((width-NewWidth)/2+WidthShift):int((width+NewWidth)/2+WidthShift),:]
#mask image with circle
#ax = fig.add_subplot(111)
circ = patches.Circle((NewWidth/2, NewHeight/2), NewWidth/2, facecolor='none')


ax4.add_patch(circ)
ax4.imshow(img, clip_path=circ, clip_on=True)


custom = patches.Circle((NewWidth/2, NewHeight/2), NewWidth/2, facecolor='none')

"""
======================
add position of line profiles
======================
"""
locationshift =(width-NewWidth)/2
origin = (NewWidth/2,NewHeight/2)
pos11 = (profile_location1-locationshift,0)             # THIS IS TOTALLY MANUAL!
pos12 = (profile_location1-locationshift,NewHeight-30)
pos21 = (profile_location2-locationshift,31)             # THIS IS TOTALLY MANUAL!
pos22 = (profile_location2-locationshift,NewHeight)


newpos11 = rotate(origin, pos11, math.radians(-profileRotation))
newpos12 = rotate(origin, pos12, math.radians(-profileRotation))
newpos21 = rotate(origin, pos21, math.radians(-profileRotation))
newpos22 = rotate(origin, pos22, math.radians(-profileRotation))

ax4.plot([newpos11[0],newpos12[0]], [newpos11[1],newpos12[1]], color= Aalto_yellow, linestyle='-', linewidth=plotlinewidth*1.5, clip_path=circ)
ax4.plot([newpos21[0],newpos22[0]], [newpos21[1],newpos12[1]], color= Aalto_yellow, linestyle='-', linewidth=plotlinewidth*1.5, clip_path=circ)

ax4.plot([0,NewWidth], [-1,-1], color= 'white', linestyle='-', linewidth=2)
ax4.plot([0,NewWidth], [NewHeight-1,NewHeight-1], color= 'white', linestyle='-', linewidth=2)

ax4.axis('off')

ax4.text(0.17,0.67,'b',color = Aalto_yellow,fontsize=label_fontsize, horizontalalignment='center', verticalalignment='center' ,transform = ax4.transAxes)
ax4.text(0.72, 0.74,'c',color = Aalto_yellow,fontsize=label_fontsize, horizontalalignment='center', verticalalignment='center',transform = ax4.transAxes)
ax4.text(0.42,0.57,'d',color = Aalto_yellow,fontsize=label_fontsize,transform = ax4.transAxes)
ax4.text(0.48,0.52,'e',color = Aalto_yellow,fontsize=label_fontsize,transform = ax4.transAxes)
ax4.text(0.55,0.38,'f',color = Aalto_yellow,fontsize=label_fontsize,transform = ax4.transAxes)

###
#Add line profiles
###

"""
======================
185px data
======================
"""

Xshift185px = 2350
Xshift185pxsim = 2340

exp_angle = np.loadtxt('angle_profiles/angle_profile_185px.txt',skiprows=headerrows)
Ku1e3angle = np.loadtxt('angle_profiles/185px_sims/185pixels_reduced_widthsKip_1e3_profile.txt',skiprows=headerrows)
Ku2e3angle = np.loadtxt('angle_profiles/185px_sims/185pixels_reduced_widthsKip_2e3_profile.txt',skiprows=headerrows)

Ku1e3angle_mod = Ku1e3angle
Ku1e3angle_mod[:,0] = np.max(Ku1e3angle[:,0])+np.min(Ku1e3angle[:,0])-Ku1e3angle[:,0]+1750
Ku1e3angle_mod[:,1] = 90-Ku1e3angle[:,1]

Ku2e3angle_mod = Ku2e3angle
Ku2e3angle_mod[:,0] = np.max(Ku2e3angle[:,0])+np.min(Ku2e3angle[:,0])-Ku2e3angle[:,0]+1750
Ku2e3angle_mod[:,1] = 90-Ku2e3angle[:,1]

N = exp_angle.shape[0]
exp_angle = np.append(exp_angle, np.zeros((N,1)), axis=1)
idx = np.argwhere(np.diff(np.sign(exp_angle[:,1] - exp_angle[:,4]))).flatten()
for i in idx:
    ax5.axvline(x=(exp_angle[i,0]-Xshift185px)/1000, color = Aalto_yellow, linewidth=plotlinewidth)
    
ax5.fill_between((exp_angle[:,0]-Xshift185px)/1000, exp_angle[:,2], exp_angle[:,3], color = Aalto_grey, alpha = 0.7)
ax5.plot((exp_angle[:,0]-Xshift185px)/1000,exp_angle[:,1],color = 'black')
ax5.plot((Ku1e3angle_mod[:,0]-Xshift185pxsim)/1000,Ku1e3angle_mod[:,1],color = Aalto_red)
ax5.plot((Ku2e3angle_mod[:,0]-Xshift185pxsim)/1000,Ku2e3angle_mod[:,1],color = Aalto_turqoise)



#relabel ticks of axes:
ax5.set_xlim(0, 6)
ax5.xaxis.set_major_locator(plt.MultipleLocator(2))
ax5.set_ylim(-35, 35)
ax5.yaxis.set_major_locator(plt.MultipleLocator(15))

#ax5.set_xlabel('$x$ [$\mathrm{\mu}$m]', fontsize = label_fontsize, labelpad=xlabelpad)
ax5.set_ylabel('$\phi$ [deg]', fontsize = label_fontsize, labelpad=ylabelpad)

ax5.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=True, labeltop=False, labelleft=True, labelright=False, direction='in', labelsize=tick_fontsize)

#ax5.text(0.4,25,'1e3 J/m$^3$',color = Aalto_red,fontsize=label_fontsize)
#ax5.text(0.4,18,'2e3 J/m$^3$',color = Aalto_turqoise,fontsize=label_fontsize)

ax5.text(2,-31,'d',color = Aalto_yellow,fontsize=label_fontsize)
ax5.text(2.75,-31,'e',color = Aalto_yellow,fontsize=label_fontsize)
ax5.text(4.2,-31,'f',color = Aalto_yellow,fontsize=label_fontsize)

"""
======================
515px data
======================
"""

Xshift515px = 2350
Xshift515pxsim = 2425

exp_angle = np.loadtxt('angle_profiles/angle_profile_515px.txt',skiprows=headerrows)
Ku1e3angle = np.loadtxt('angle_profiles/515px_sims/515pixels_reduced_widthsKip_1e3_profile.txt',skiprows=headerrows)
Ku2e3angle = np.loadtxt('angle_profiles/515px_sims/515pixels_reduced_widthsKip_2e3_profile.txt',skiprows=headerrows)

Ku1e3angle_mod = Ku1e3angle
Ku1e3angle_mod[:,0] = np.max(Ku1e3angle[:,0])+np.min(Ku1e3angle[:,0])-Ku1e3angle[:,0]+1750
Ku1e3angle_mod[:,1] = 90-Ku1e3angle[:,1]

Ku2e3angle_mod = Ku2e3angle
Ku2e3angle_mod[:,0] = np.max(Ku2e3angle[:,0])+np.min(Ku2e3angle[:,0])-Ku2e3angle[:,0]+1750
Ku2e3angle_mod[:,1] = 90-Ku2e3angle[:,1]

N = exp_angle.shape[0]
exp_angle = np.append(exp_angle, np.zeros((N,1)), axis=1)
idx = np.argwhere(np.diff(np.sign(exp_angle[:,1] - exp_angle[:,4]))).flatten()
for i in idx:
    ax6.axvline(x=(exp_angle[i,0]-Xshift515px)/1000, color = Aalto_yellow, linewidth=plotlinewidth, linestyle = '-')

ax6.fill_between((exp_angle[:,0]-Xshift515px)/1000, exp_angle[:,2], exp_angle[:,3], color = Aalto_grey, alpha = 0.7)
ax6.plot((exp_angle[:,0]-Xshift515px)/1000,exp_angle[:,1],color = 'black')
ax6.plot((Ku1e3angle_mod[:,0]-Xshift515pxsim)/1000,Ku1e3angle_mod[:,1],color = Aalto_red)
ax6.plot((Ku2e3angle_mod[:,0]-Xshift515pxsim)/1000,Ku2e3angle_mod[:,1],color = Aalto_turqoise)


#relabel ticks of axes:
ax6.set_xlim(0, 6)
ax6.xaxis.set_major_locator(plt.MultipleLocator(2))
ax6.set_ylim(-35, 35)
ax6.yaxis.set_major_locator(plt.MultipleLocator(15))

ax6.set_xlabel('$position$ [$\mathrm{\mu}$m]', fontsize = label_fontsize, labelpad=xlabelpad)
ax6.set_ylabel('$\phi$ [deg]', fontsize = label_fontsize, labelpad=ylabelpad)

ax6.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=True, labeltop=False, labelleft=True, labelright=False, direction='in', labelsize=tick_fontsize)

ax6.text(1.87,-30,'d',color = Aalto_yellow,fontsize=label_fontsize)
ax6.text(2.7,-33,'e',color = Aalto_yellow,fontsize=label_fontsize)
ax6.text(4.3,-33,'f',color = Aalto_yellow,fontsize=label_fontsize)



ax1.text(0.495,2,r'$1\times10^3$ J/m$^3$',color = Aalto_red,fontsize=label_fontsize)
ax1.text(0.45,16,r'$2\times10^3$ J/m$^3$',color = Aalto_turqoise,fontsize=label_fontsize)


"""
add panel labels
"""
axlabels = fig.add_axes([0,0,1,1],label ='labels')
axlabels.axis('off')
axlabels.patch.set_alpha(0.0)
axlabels.text(0.0,0.96,'(a)',fontsize=label_fontsize)
axlabels.text(0.0,0.64,'(b)',fontsize=label_fontsize)
axlabels.text(0.0,0.32,'(c)',fontsize=label_fontsize)
axlabels.text(0.95,0.96,'(d)',fontsize=label_fontsize)
axlabels.text(0.95,0.64,'(e)',fontsize=label_fontsize)
axlabels.text(0.95,0.32,'(f)',fontsize=label_fontsize)
axlabels.tick_params(bottom=False, top=False, left=False, right=False, labelbottom=False, labeltop=False, labelleft=False, labelright=False)




"""
======================
show image
======================
"""
plt.savefig('ani_fit.pdf')
plt.close()
#plt.show()


