import numpy as np
from scipy.interpolate import griddata
import matplotlib.pyplot as plt
import numpy.ma as ma
from numpy.random import uniform, seed
import os
from matplotlib.colors import hsv_to_rgb
import colorsys
from skimage import measure
import matplotlib.ticker as ticker
import matplotlib.font_manager as fm
from colour import Color
import matplotlib.gridspec as gridspec
import math
from matplotlib.pyplot import figure, show, grid, tight_layout
from PIL import Image
import scipy.ndimage
import matplotlib.image as mpimg
import matplotlib.patches as patches




"""
======================
Definitions
======================
"""
def cm2inch(value):
    return value/2.54

Aalto_yellow = '#FECB00' 
Aalto_orange = '#FF7900' #
Aalto_red = '#ED2939' 
Aalto_violet = '#B1059D' #
Aalto_purple = '#6639B7'
Aalto_blue = '#0065BD' #
Aalto_turqoise = '#00A8B4'
Aalto_green = '#009B3A' #
Aalto_light_green = '#69BE28'
Aalto_grey = '#928B81'

nmpp = 21.6 #This is nm per image pixel

"""
2. define parameters:
"""

headerrows = 3


label_fontsize = 8
tick_fontsize = 6

contourlinewidth = 0.5
plotlinewidth = 1
symbolsize = 3

x1 = 0.1
x2 = 0.51
y1 = 0.55
y2 = 0.1



widthw =0.39
heightw = 0.44

# define panel size and position:
width = 8.6
heightratio = 0.75
figuresize=(cm2inch(width), cm2inch(heightratio*width))

panel_1 = [x2, y1, widthw, heightw]
panel_2 = [x1, y1, widthw, heightw]
panel_3 = [x2, y2, widthw, heightw]
panel_4 = [x1, y2, widthw, heightw]

rlabelpad = -1
thetalabelpad = -3
xlabelpad = 1
ylabelpad = 0


"""
create image:
"""
fig = plt.figure(num=None, figsize=figuresize, dpi=80, facecolor='w', edgecolor='k')

ax1 = fig.add_axes(panel_1, label ='1')
ax2 = fig.add_axes(panel_2, label ='2')
ax3 = fig.add_axes(panel_3, label ='3')
ax4 = fig.add_axes(panel_4, label ='4')


"""
======================
import data
======================
"""


Ku1p5e3_60C_w0p5_Energy = np.loadtxt('t_odts_60C_1p5e3_w0p5.txt',skiprows=2)
Ku1p5e3_60C_w1_Energy = np.loadtxt('t_odts_60C_1p5e3_w1.txt',skiprows=2)
Ku1p5e3_60C_w2_Energy = np.loadtxt('t_odts_60C_1p5e3_w2.txt',skiprows=2)

Ku1p5e3_120U_w0p5_Energy = np.loadtxt('t_odts_120U_1p5e3_w0p5.txt',skiprows=2)
Ku1p5e3_120U_w1_Energy = np.loadtxt('t_odts_120U_1p5e3_w1.txt',skiprows=2)
Ku1p5e3_120U_w2_Energy = np.loadtxt('t_odts_120U_1p5e3_w2.txt',skiprows=2)

Ku1p5e3_60C_w0p5_DW = np.loadtxt('t_DW_data_60C_Ku1p5e3_w0p5.txt',skiprows=2)
Ku1p5e3_60C_w1_DW = np.loadtxt('t_DW_data_60C_Ku1p5e3_w1.txt',skiprows=2)
Ku1p5e3_60C_w2_DW = np.loadtxt('t_DW_data_60C_Ku1p5e3_w2.txt',skiprows=2)

Ku1p5e3_120U_w0p5_DW = np.loadtxt('t_DW_data_120U_Ku1p5e3_w0p5.txt',skiprows=2)
Ku1p5e3_120U_w1_DW = np.loadtxt('t_DW_data_120U_Ku1p5e3_w1.txt',skiprows=2)
Ku1p5e3_120U_w2_DW = np.loadtxt('t_DW_data_120U_Ku1p5e3_w2.txt',skiprows=2)



Ku1p5e3_60C_t1_Energy = np.loadtxt('w_odts_60C_1p5e3_t_1.txt',skiprows=2)
Ku1p5e3_60C_t3_Energy = np.loadtxt('w_odts_60C_1p5e3_t_3.txt',skiprows=2)
Ku1p5e3_60C_t10_Energy = np.loadtxt('w_odts_60C_1p5e3_t_10.txt',skiprows=2)

Ku1p5e3_120U_t1_Energy = np.loadtxt('w_odts_120U_1p5e3_t_1.txt',skiprows=2)
Ku1p5e3_120U_t3_Energy = np.loadtxt('w_odts_120U_1p5e3_t_3.txt',skiprows=2)
Ku1p5e3_120U_t10_Energy = np.loadtxt('w_odts_120U_1p5e3_t_10.txt',skiprows=2)

Ku1p5e3_60C_t1_DW = np.loadtxt('w_DW_data_60C_1p5e3_t_1.txt',skiprows=2)
Ku1p5e3_60C_t3_DW = np.loadtxt('w_DW_data_60C_1p5e3_t_3.txt',skiprows=2)
Ku1p5e3_60C_t10_DW = np.loadtxt('w_DW_data_60C_1p5e3_t_10.txt',skiprows=2)

Ku1p5e3_120U_t1_DW = np.loadtxt('w_DW_data_120U_1p5e3_t_1.txt',skiprows=2)
Ku1p5e3_120U_t3_DW = np.loadtxt('w_DW_data_120U_1p5e3_t_3.txt',skiprows=2)
Ku1p5e3_120U_t10_DW = np.loadtxt('w_DW_data_120U_1p5e3_t_10.txt',skiprows=2)

"""
======================
Calculate Energy densities
======================
"""
Ku1p5e3_60C_w0p5_eDens  = np.copy(Ku1p5e3_60C_w0p5_Energy)
Ku1p5e3_60C_w0p5_eDens[:,2] = Ku1p5e3_60C_w0p5_Energy[:,2]/(1e-6*1e-6*Ku1p5e3_60C_w0p5_Energy[:,0]*1e-9)
Ku1p5e3_60C_w1_eDens  = np.copy(Ku1p5e3_60C_w1_Energy)
Ku1p5e3_60C_w1_eDens[:,2] = Ku1p5e3_60C_w1_Energy[:,2]/(2e-6*1e-6*Ku1p5e3_60C_w1_Energy[:,0]*1e-9)
Ku1p5e3_60C_w2_eDens  = np.copy(Ku1p5e3_60C_w2_Energy)
Ku1p5e3_60C_w2_eDens[:,2] = Ku1p5e3_60C_w2_Energy[:,2]/(4e-6*1e-6*Ku1p5e3_60C_w2_Energy[:,0]*1e-9)

Ku1p5e3_120U_w0p5_eDens  = np.copy(Ku1p5e3_120U_w0p5_Energy)
Ku1p5e3_120U_w0p5_eDens[:,2] = Ku1p5e3_120U_w0p5_Energy[:,2]/(1e-6*1e-6*Ku1p5e3_120U_w0p5_Energy[:,0]*1e-9)
Ku1p5e3_120U_w1_eDens  = np.copy(Ku1p5e3_120U_w1_Energy)
Ku1p5e3_120U_w1_eDens[:,2] = Ku1p5e3_120U_w1_Energy[:,2]/(2e-6*1e-6*Ku1p5e3_120U_w1_Energy[:,0]*1e-9)
Ku1p5e3_120U_w2_eDens  = np.copy(Ku1p5e3_120U_w2_Energy)
Ku1p5e3_120U_w2_eDens[:,2] = Ku1p5e3_120U_w2_Energy[:,2]/(4e-6*1e-6*Ku1p5e3_120U_w2_Energy[:,0]*1e-9)

eDens_fraction_w0p5 = Ku1p5e3_60C_w0p5_eDens[:,2]/Ku1p5e3_120U_w0p5_eDens[:,2]
eDens_fraction_w1 = Ku1p5e3_60C_w1_eDens[:,2]/Ku1p5e3_120U_w1_eDens[:,2]
eDens_fraction_w2 = Ku1p5e3_60C_w2_eDens[:,2]/Ku1p5e3_120U_w2_eDens[:,2]

e_fraction_t1 = Ku1p5e3_60C_t1_Energy[:,2]/Ku1p5e3_120U_t1_Energy[:,2]
e_fraction_t3 = Ku1p5e3_60C_t3_Energy[:,2]/Ku1p5e3_120U_t3_Energy[:,2]
e_fraction_t10 = Ku1p5e3_60C_t10_Energy[:,2]/Ku1p5e3_120U_t10_Energy[:,2]

SpinRot_fraction_w0p5 = Ku1p5e3_60C_w0p5_DW[:,1]/Ku1p5e3_120U_w0p5_DW[:,1]
SpinRot_fraction_w1 = Ku1p5e3_60C_w1_DW[:,1]/Ku1p5e3_120U_w1_DW[:,1]
SpinRot_fraction_w2 = Ku1p5e3_60C_w2_DW[:,1]/Ku1p5e3_120U_w2_DW[:,1]

SpinRot_fraction_t1 = Ku1p5e3_60C_t1_DW[:,1]/Ku1p5e3_120U_t1_DW[:,1]
SpinRot_fraction_t3 = Ku1p5e3_60C_t3_DW[:,1]/Ku1p5e3_120U_t3_DW[:,1]
SpinRot_fraction_t10 = Ku1p5e3_60C_t10_DW[:,1]/Ku1p5e3_120U_t10_DW[:,1]

"""
======================
plot data
======================
"""

ax1.semilogy(Ku1p5e3_60C_w0p5_DW[:,0], SpinRot_fraction_w0p5, color = Aalto_orange, linewidth=plotlinewidth)
ax1.semilogy(Ku1p5e3_60C_w1_DW[:,0], SpinRot_fraction_w1, color = Aalto_violet, linewidth=plotlinewidth)
ax1.semilogy(Ku1p5e3_60C_w2_DW[:,0], SpinRot_fraction_w2, color = Aalto_blue, linewidth=plotlinewidth)

ax2.semilogy(Ku1p5e3_60C_t1_DW[1:,0]/1000, SpinRot_fraction_t1[1:], color = Aalto_purple, linewidth=plotlinewidth)
ax2.semilogy(Ku1p5e3_60C_t3_DW[1:,0]/1000, SpinRot_fraction_t3[1:], color = Aalto_turqoise, linewidth=plotlinewidth)
ax2.semilogy(Ku1p5e3_60C_t10_DW[1:,0]/1000, SpinRot_fraction_t10[1:], color = Aalto_light_green, linewidth=plotlinewidth)


#ax3.semilogy([0,20],[1,1],'-', color =  Aalto_grey, linewidth=plotlinewidth)
ax3.semilogy([3,3],[0.1,2],'-', color =  Aalto_grey, linewidth=plotlinewidth)
ax3.semilogy(Ku1p5e3_60C_w0p5_eDens[:,0], eDens_fraction_w0p5, color = Aalto_orange, linewidth=plotlinewidth)
ax3.semilogy(Ku1p5e3_60C_w1_eDens[:,0], eDens_fraction_w1, color = Aalto_violet, linewidth=plotlinewidth)
ax3.semilogy(Ku1p5e3_60C_w2_eDens[:,0], eDens_fraction_w2, color = Aalto_blue, linewidth=plotlinewidth)

#ax4.semilogy([0,8],[1,1],'-', color =  Aalto_grey, linewidth=plotlinewidth)
ax4.semilogy(Ku1p5e3_60C_t1_Energy[1:,0]/1000, e_fraction_t1[1:], color = Aalto_purple, linewidth=plotlinewidth)
ax4.semilogy(Ku1p5e3_60C_t3_Energy[1:,0]/1000, e_fraction_t3[1:], color = Aalto_turqoise, linewidth=plotlinewidth)
ax4.semilogy(Ku1p5e3_60C_t10_Energy[1:,0]/1000, e_fraction_t10[1:], color = Aalto_light_green, linewidth=plotlinewidth)

"""
======================
plot design
======================
"""
#set limits
tmin = 0 
tmax = 15
tintn = 10
tintw = 5
Cangmin = 0
Cangmax = 65
Cangint = 15
Uangmin = 60
Uangmax = 125
Uangint = 15
Uangtickmin = 60
Eratmin = 0.38
Eratmax = 1.48

Erat2min = 0.28
Erat2max = 1.8

SRmin = 0.1
SRmax = 0.6

SR2min = 0.1
SR2max = 0.6

wmin = 0 
wmax = 5
wintn = 2
wintw = 2

ax1.xaxis.set_ticks(np.arange(tmin, tmax+1, tintw))
ax1.set_xlim(tmin, tmax)
ax3.xaxis.set_ticks(np.arange(tmin, tmax+1, tintw))
ax3.set_xlim(tmin, tmax)

ax2.xaxis.set_ticks(np.arange(wmin, wmax+1, wintw))
ax2.set_xlim(wmin, wmax)
ax4.xaxis.set_ticks(np.arange(wmin, wmax+1, wintw))
ax4.set_xlim(wmin, wmax)

ax1.set_yticks([0.125, 0.25, 0.5])
ax1.set_yticklabels([r'$\dfrac{1}{8}$', r'$\dfrac{1}{4}$', r'$\dfrac{1}{2}$'])

ax2.set_yticks([0.125, 0.25, 0.5])
ax2.set_yticklabels([r'$\dfrac{1}{8}$', r'$\dfrac{1}{4}$', r'$\dfrac{1}{2}$'])


ax3.set_yticks([0.5,0.75,1,1.25])
ax3.set_yticklabels([r'$\dfrac{1}{2}$',r'$\dfrac{3}{4}$','1',r'$\dfrac{5}{4}$'])

ax4.set_yticks([0.5,1,1.5])
ax4.set_yticklabels([r'$\dfrac{1}{2}$','1',r'$\dfrac{3}{2}$'])

ax1.set_ylim(SRmin, SRmax)

ax2.set_ylim(SR2min, SR2max)

ax3.set_ylim(Eratmin, Eratmax)

ax4.set_ylim(Erat2min, Erat2max)

#label axes

ax1.set_ylabel(r'log $\mathcal{Q}$', fontsize = label_fontsize, labelpad=ylabelpad)
ax2.set_ylabel(r'log $\mathcal{Q}$', fontsize = label_fontsize, labelpad=ylabelpad)

ax3.set_xlabel('$t$ [nm] ', fontsize = label_fontsize, labelpad=xlabelpad)
ax3.set_ylabel(r'log $E_{c} \,/\, E_{u}$', fontsize = label_fontsize, labelpad=ylabelpad+2)

ax4.set_xlabel(r'$w$ [$\mu$m] ', fontsize = label_fontsize, labelpad=xlabelpad)
ax4.set_ylabel(r'log $E_{c} \,/\, E_{u}$', fontsize = label_fontsize, labelpad=ylabelpad+2)

ax1.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=False, labeltop=False, labelleft=False, labelright=True, direction='in', labelsize=tick_fontsize)
ax1.minorticks_off()

ax2.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=False, labeltop=False, labelleft=True, labelright=False, direction='in', labelsize=tick_fontsize)
ax2.minorticks_off()

ax3.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=True, labeltop=False, labelleft=False, labelright=True, direction='in', labelsize=tick_fontsize)
ax3.minorticks_off()

ax4.tick_params(bottom=True, top=True, left=True, right=True, labelbottom=True, labeltop=False, labelleft=True, labelright=False, direction='in', labelsize=tick_fontsize)
ax4.minorticks_off()

ax1.yaxis.set_label_position("right")
ax3.yaxis.set_label_position("right")
"""
======================
add labels
======================
"""



ax1.text(0.09,0.1,'$w = 0.5 \, \mu$m',color = Aalto_orange, fontsize=label_fontsize,transform=ax1.transAxes)
ax1.text(0.09,0.2,'$w = 1 \, \mu$m',color = Aalto_violet, fontsize=label_fontsize,transform=ax1.transAxes)
ax1.text(0.09,0.3,'$w = 2 \, \mu$m',color = Aalto_blue, fontsize=label_fontsize,transform=ax1.transAxes)

ax2.text(0.3,0.1,'$t = 1 \,$nm',color = Aalto_purple, fontsize=label_fontsize,transform=ax2.transAxes)
ax2.text(0.3,0.2,'$t = 3 \,$nm',color = Aalto_turqoise, fontsize=label_fontsize,transform=ax2.transAxes)
ax2.text(0.3,0.3,'$t = 10 \,$nm',color = Aalto_light_green, fontsize=label_fontsize,transform=ax2.transAxes)


"""
add panel labels
"""
axlabels = fig.add_axes([0,0,1,1],label ='labels')
axlabels.axis('off')
axlabels.patch.set_alpha(0.0)
axlabels.text(0.0,0.95,'(a)',fontsize=label_fontsize)
axlabels.text(0.95,0.95,'(b)',fontsize=label_fontsize)
axlabels.text(0.0,0.49,'(c)',fontsize=label_fontsize)
axlabels.text(0.95,0.49,'(d)',fontsize=label_fontsize)
axlabels.tick_params(bottom=False, top=False, left=False, right=False, labelbottom=False, labeltop=False, labelleft=False, labelright=False)



"""
======================
show Figure
======================
"""
plt.savefig('Figure_t-dep.pdf')
plt.close()
#plt.show()


